'use strict';

const vite = require('vite');
const core = require('@babel/core');
const replaceAsync = require('string-replace-async');

function _interopDefaultCompat (e) { return e && typeof e === 'object' && 'default' in e ? e.default : e; }

const replaceAsync__default = /*#__PURE__*/_interopDefaultCompat(replaceAsync);

const logger = vite.createLogger(void 0, { prefix: "vite-plugin-conditional-compile" });

var __defProp = Object.defineProperty;
var __defNormalProp = (obj, key, value) => key in obj ? __defProp(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __publicField = (obj, key, value) => {
  __defNormalProp(obj, typeof key !== "symbol" ? key + "" : key, value);
  return value;
};
class Context {
  constructor(userOptions) {
    __publicField(this, "ctx", {});
    __publicField(this, "code", "");
    __publicField(this, "filepath", "");
    __publicField(this, "root", "");
    __publicField(this, "enable_source_maps", false);
    __publicField(this, "sourceMapFileNameReg", /\.[tj]sx?$/);
    __publicField(this, "tsReg", /\.tsx?$/);
    __publicField(this, "pattern_reg", /(?:\/\/|\{?\s*\/\*|<!--)\s*#\s*if\s\[([\w!=&|()'"\s]+)\]\s*(?:\*\/\s*\}?|-->)?([\s\S]+?)(?:\/\/|\{?\s*\/\*|<!--)\s*#\s*endif\s*(?:\*\/\s*\}?|-->)?/g);
    __publicField(this, "match_group_reg", /.*(#\s*(?:if|elif|else|endif))\s?(?:\[([\w !=&|()'"]*)\])*(?:\s*\*\/\s*\}?|\s*-->)?/g);
    this.ctx = userOptions;
  }
  set_env(env) {
    this.ctx.env = { ...env, ...this.ctx.env };
  }
  set_root(root) {
    this.root = root;
  }
  set_enable_source_maps(enable) {
    this.enable_source_maps = enable;
  }
  resolve_options(userOptions) {
    return {
      include: ["**/*"],
      exclude: [/[\\/]node_modules[\\/]/, /[\\/]\.git[\\/]/],
      ...userOptions
    };
  }
  evaluate_ast(node) {
    try {
      if (node.type === "BinaryExpression") {
        const { left, operator, right } = node;
        if (left.type === "Identifier" && ["==", "!="].includes(operator)) {
          if (right.type === "Identifier") {
            if (operator === "==") {
              return this.ctx.env?.[left.name] == right.name;
            }
            return this.ctx.env?.[left.name] != right.name;
          }
          if (right.type === "NumericLiteral") {
            if (operator === "==") {
              return this.ctx.env?.[left.name] == right.value;
            }
            return this.ctx.env?.[left.name] != right.value;
          }
        } else {
          logger.error(
            `evaluate ast error: This expression is not supported: ${left.type}${operator}${right.type}`
          );
          return false;
        }
      }
      if (node.type === "Identifier") {
        return this.ctx.env?.[node.name] === true;
      }
      if (node.type === "NumericLiteral") {
        return this.ctx.env?.[node.value] === true;
      }
      if (node.type === "UnaryExpression") {
        if (node.operator === "!") {
          if (node.argument.type === "Identifier") {
            return !this.ctx.env?.[node.argument.name];
          }
          if (node.argument.type === "UnaryExpression") {
            return !this.evaluate_ast(node.argument);
          }
          if (node.argument.type === "BinaryExpression") {
            return !this.evaluate_ast(node.argument);
          }
          logger.error(
            `evaluate ast error: This expression is not supported: ${node.type}:${node.operator}`
          );
          return false;
        } else {
          logger.error(
            `evaluate ast error: This expression is not supported: ${node.type}:${node.operator}`
          );
          return false;
        }
      }
      if (node.type === "LogicalExpression") {
        const { left, operator, right } = node;
        if (operator === "&&") {
          return this.evaluate_ast(left) && this.evaluate_ast(right);
        }
        if (operator === "||") {
          return this.evaluate_ast(left) || this.evaluate_ast(right);
        }
        if (operator === "??") {
          return this.evaluate_ast(left) ?? this.evaluate_ast(right);
        }
      }
      logger.error(`evaluate ast error: This expression is not supported: ${node.type}`);
      return false;
    } catch (error) {
      logger.error(`evaluate ast error: ${error}`);
      return false;
    }
  }
  async resolve_conditional_async(conditional) {
    let res = false;
    try {
      const ast = await core.parseAsync(conditional.replace(/([^=!])=([^=])/g, "$1==$2"));
      if (!ast) {
        return false;
      }
      core.traverse(ast, {
        Program: ({ node }) => {
          res = this.evaluate_ast(
            node.body[0].expression
          );
        }
      });
    } catch (error) {
      logger.error(`${this.filepath}: babel parse error: ${error}`);
      return false;
    }
    return res;
  }
  async resolve_comment_async() {
    return replaceAsync__default(this.code, this.pattern_reg, async (matchCode) => {
      const codeBlocks = matchCode.split(this.match_group_reg).filter((v) => v !== "");
      while (codeBlocks.length) {
        const [flag, conditional, block] = codeBlocks.splice(0, 3);
        const real_flag = flag.replace(/\s/g, "");
        if (real_flag === "#if" || real_flag === "#elif") {
          if (await this.resolve_conditional_async(conditional)) {
            return block;
          }
        } else if (real_flag === "#else") {
          return block;
        }
      }
      return "";
    });
  }
  deprecated_transform(code) {
    if (!("isDebug" in this.ctx)) {
      this.ctx.isDebug = !!this.ctx.env?.DEV;
    }
    try {
      const js = this.ctx?.changeSource?.(code) ?? code;
      const replaceMatched = () => {
        const REG = /\/\*\s*IF(DEBUG|TRUE_\w+)(?:\s*\*\/)?([\s\S]+?)(?:\/\*\s*)?FI\1\s*\*\//g;
        return js.replace(REG, (_, $1, $2) => {
          let isKeep = false;
          if ($1 === "DEBUG") {
            isKeep = this.ctx.isDebug;
          } else {
            isKeep = !!this.ctx.expand?.[$1.slice(5)];
          }
          return isKeep ? $2 : "";
        });
      };
      return replaceMatched();
    } catch (error) {
      throw error;
    }
  }
  async transformAsync(code, id) {
    if (id.includes("/node_modules/"))
      return;
    const [filepath] = id.split("?");
    this.code = code;
    this.filepath = filepath;
    const options = this.resolve_options({ include: this.ctx.include, exclude: this.ctx.exclude });
    const filter = vite.createFilter(options.include, options.exclude);
    let transformedCode = "";
    if (!filter(this.filepath)) {
      transformedCode = this.deprecated_transform(code);
    } else {
      transformedCode = this.deprecated_transform(await this.resolve_comment_async());
    }
    if (!this.enable_source_maps || !this.sourceMapFileNameReg.test(this.filepath)) {
      return {
        code: transformedCode,
        map: null
      };
    }
    try {
      const parserPlugins = [];
      if (!this.filepath.endsWith(".ts")) {
        parserPlugins.push("jsx");
      }
      if (this.tsReg.test(this.filepath)) {
        parserPlugins.push("typescript");
      }
      const babelTransformed = await core.transformAsync(transformedCode, {
        root: this.root,
        filename: id,
        sourceFileName: this.filepath,
        parserOpts: {
          sourceType: "module",
          allowAwaitOutsideFunction: true,
          plugins: parserPlugins
        },
        generatorOpts: {
          decoratorsBeforeExport: true
        },
        sourceMaps: true
      });
      return {
        code: babelTransformed?.code ?? void 0,
        map: babelTransformed?.map
      };
    } catch (error) {
      logger.error(`${this.filepath}: generate source maps error: ${error}`);
      return {
        code: transformedCode,
        map: null
      };
    }
  }
}
const create_context = (userOptions) => {
  return new Context(userOptions);
};

const vitePluginConditionalCompile = (userOptions = {}) => {
  const ctx = create_context(userOptions);
  return {
    name: "vite-plugin-conditional-compile",
    enforce: "pre",
    configResolved(config) {
      ctx.set_env({
        ...vite.loadEnv(config.mode ?? process.env.NODE_ENV, process.cwd(), ""),
        ...config.env
      });
      ctx.set_root(config.root);
      ctx.set_enable_source_maps(
        config.isProduction && !!config.build.sourcemap || !config.isProduction
      );
    },
    transform(code, id) {
      return ctx.transformAsync(code, id);
    }
  };
};

module.exports = vitePluginConditionalCompile;
